function classifyResult = challengeSave(recordName)
%
global bigFeatureVector;
% Sample entry for the 2016 PhysioNet/CinC Challenge.
%
% INPUTS:
% recordName: string specifying the record name to process
%
% OUTPUTS:
% classifyResult: integer value where
%                     1 = abnormal recording
%                    -1 = normal recording
%                     0 = unsure (too noisy)
%
% To run your entry on the entire training set in a format that is
% compatible with PhysioNet's scoring enviroment, run the script
% generateValidationSet.m
%
% The challenge function requires that you have downloaded the challenge
% data 'training_set' in a subdirectory of the current directory.
%    http://physionet.org/physiobank/database/challenge/2016/
%
% This dataset is used by the generateValidationSet.m script to create
% the annotations on your training set that will be used to verify that
% your entry works properly in the PhysioNet testing environment.
%
%
% Version 1.0
%
%
% Written by: Chengyu Liu, Fubruary 21 2016
%             chengyu.liu@emory.edu
%
% Last modified by: Chengyu Liu, April 3 2016
%             Note: using 'audioread' to replace the 'wavread' function to load the .wav data.
%
%

%% Load the trained parameter matrices for Springer's HSMM model.
% The parameters were trained using 409 heart sounds from MIT heart
% sound database, i.e., recordings a0001-a0409.
load('Springer_B_matrix.mat');
load('Springer_pi_vector.mat');
load('Springer_total_obs_distribution.mat');

%% Load data and resample data
springer_options   = default_Springer_HSMM_options;
[PCG, Fs1]         = audioread([recordName '.wav']);  % load data
PCG_resampled      = resample(PCG,springer_options.audio_Fs,Fs1); % resample to springer_options.audio_Fs (1000 Hz)
N=60; sr = 1000;
Wn = 45*2/sr;
b1 = fir1(N,Wn,'low',hamming(N+1));
Wn = [45*2/sr, 80*2/sr];
b2 = fir1(N,Wn,hamming(N+1));
Wn = [200*3/sr, 200*4/sr];
b3 = fir1(N,Wn,hamming(N+1));
Wn = 200*4/sr;
b4 = fir1(N,Wn,'high',hamming(N+1));
Wn = [200*2/sr, 200*3/sr];
b5 = fir1(N,Wn,hamming(N+1));
Indexes=[1 2 3];
PCG1 = filtfilt(b1,1,PCG_resampled);
[~,featuresA1]=findPCGpeaksNewFeatures2IndexDiffsEntropyFFT(PCG1,Indexes);
PCG2 = filtfilt(b2,1,PCG_resampled);
[~,featuresA2]=findPCGpeaksNewFeatures2IndexDiffsEntropyFFT(PCG2,Indexes);
PCG3 = filtfilt(b3,1,PCG_resampled);
[~,featuresA3]=findPCGpeaksNewFeatures2IndexDiffsEntropyFFT(PCG3,Indexes);
PCG4 = filtfilt(b4,1,PCG_resampled);
[~,featuresA4]=findPCGpeaksNewFeatures2IndexDiffsEntropyFFT(PCG4,Indexes);
PCG5 = filtfilt(b5,1,PCG_resampled);
[~,featuresA5]=findPCGpeaksNewFeatures2IndexDiffsEntropyFFT(PCG5,Indexes);
[~,featuresA]=findPCGpeaksNewFeatures2IndexDiffsEntropyFFT(PCG_resampled,Indexes);

pqrst=10;
% featureVector1=zeros(1,pqrst*90);
[~,featureVector1] = SelectUpdatedNumberNoSeg(PCG_resampled,pqrst,[]); %this is in the ECG folder
featureVector=[featureVector1',featuresA,featuresA1,featuresA2,featuresA3,featuresA4,featuresA5];
bigFeatureVector=[bigFeatureVector;featureVector];
classifyResult=combineEnsembles(featureVector);