function model = team_training_code1(input_directory,output_directory) % train_PCG_classifier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Train PCG classifiers and obtain the models
% Inputs:
% 1. input_directory
% 2. output_directory
%
% Outputs:
% 1. model: trained model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Find text files
patient_files=dir(fullfile(input_directory,'*.txt'));
patient_files={patient_files.name};
patient_files=sort(patient_files); % To help debugging
num_patient_files=length(patient_files);

fprintf('Loading data for %d patients...\n', num_patient_files)

% Extract classes from data
classes={};
nfilesTotal=[];
Strs=['AV';'MV';'TV';'PV';'Ph'];

PatientsAV = cell(942*1);
PatientsMV=cell(942*1);
PatientsTV=cell(942*1);
PatientsPV=cell(942*1);
PatientsPh=cell(942*1);

OutsAV=cell(942*1);
OutsMV=cell(942*1);
OutsTV=cell(942*1);
OutsPV=cell(942*1);

OutsPh=cell(942*1);
featuresAV=cell(942*1);
featuresMV=cell(942*1);
featuresTV=cell(942*1);
featuresPV=cell(942*1);
featuresPh=cell(942*1);
tic
kav=0;
kmv=0;
ktv=0;
kph=0;
kpv=0;
parfor j=1:num_patient_files
    j
    strName=[patient_files{j}(1:end-4),'*.wav'];
    
    current_header=get_header(fullfile(input_directory,patient_files{j}));
    current_recordings=load_recordings(input_directory,current_header);
    
    current_features=get_features(current_header, current_recordings);
    
    %     cpatient_file=dir(fullfile(input_directory);
    cpatient_files=dir(fullfile(input_directory,strName));
    nfiles=length(cpatient_files);
    current_class=get_class(fullfile(input_directory,patient_files{j}));
    current_class
    zclass=0;
    if strcmp(current_class,'Present')==1
        zclass=1;
    else
        if strcmp(current_class,'Unknown')==1
            zclass=2;
        end
    end
    for k=1:nfiles
        cfile=[input_directory,cpatient_files(k).name];
        features=challenge(cfile);
        if strfind(cpatient_files(k).name,'AV')
            kav=kav+1;
            featuresAV{j}=[features,current_features];
            OutsAV{j}=zclass;
            PatientsAV{j}=str2num( patient_files{j}(1:end-4));
        end
        if strfind(cpatient_files(k).name,'TV')
            ktv=ktv+1;
            featuresTV{j}=[features,current_features];
            OutsTV{j}=zclass;
            PatientsTV{j}=str2num( patient_files{j}(1:end-4));
        end
        if strfind(cpatient_files(k).name,'MV')
            kmv=kmv+1;
            featuresMV{j}=[features,current_features];
            OutsMV{j}=zclass;
            PatientsMV{j}=str2num( patient_files{j}(1:end-4));
        end
        if strfind(cpatient_files(k).name,'PV')
            kpv=kpv+1;
            featuresPV{j}=[features,current_features];
            OutsPV{j}=zclass;
            PatientsPV{j}=str2num( patient_files{j}(1:end-4));
        end
        if strfind(cpatient_files(k).name,'Ph')
            kph=kph+1;
            featuresPh{j}=[features,current_features];
            OutsPh{j}=zclass;
            PatientsPh{j}=str2num( patient_files{j}(1:end-4));
        end
        
        %         [PCG, Fs1]         = audioread([recordName '.wav']);  % load data
        %         PCG_resampled      = resample(PCG,1000,Fs1);
    end
    
    if nfiles~=4
        [cpatient_files.name," ", num2str(nfiles)]
        zz=1;
    end
    nfilesTotal=[nfilesTotal,nfiles];
    
%     classes=unique([classes current_class]);
    
end
thetime=toc;
['thetime=',num2str(thetime)]
classes=sort(classes);
num_classes=length(classes);

% Extracting features and labels
disp('Extracting features and labels...')

features=[];
labels=categorical;

for j=1:num_patient_files
    
    fprintf('%d/%d \n',j,num_patient_files)
    
    current_header=get_header(fullfile(input_directory,patient_files{j}));
    current_recordings=load_recordings(input_directory,current_header);
    
    current_features=get_features(current_header, current_recordings);
    features(j,:) = current_features(:);
    
    labels(j)=get_class(fullfile(input_directory,patient_files{j}));
    
end

%% train RF

disp('Training the model...')

model = TreeBagger(300,features,labels);
save_model(model,classes,output_directory);

disp('Done.')

end

function save_model(model,classes,output_directory) %save_PCG_model
% Save results.
filename = fullfile(output_directory,'model.mat');
save(filename,'model','classes','-v7.3');

disp('Done.')
end

function class=get_class(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Murmur'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function current_header=get_header(input_header)

current_header=fileread(input_header);
current_header=strsplit(current_header,'\n');

end

function current_recordings=load_recordings(input_directory,current_header)

recording_files=get_recording_files(current_header);

current_recordings={};

for j=1:length(recording_files)
    
    current_recordings{j}=audioread(fullfile(input_directory,strtrim(recording_files{j})));
    
end

end

function recording_files=get_recording_files(current_header)

recording_files={};

num_locations=strsplit(current_header{1},' ');
num_locations=str2double(num_locations{2});

for j=2:num_locations+1
    
    current_line=strsplit(current_header{j},' ');
    recording_files{j-1}=current_line{3};
    
end

end
