function model = team_training_codeSig(input_directory,output_directory) % train_PCG_classifier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Train PCG classifiers and obtain the models
% Inputs:
% 1. input_directory
% 2. output_directory
%
% Outputs:
% 1. model: trained model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Find text files
patient_files=dir(fullfile(input_directory,'*.txt'));
patient_files={patient_files.name};
patient_files=sort(patient_files); % To help debugging
num_patient_files=length(patient_files);

fprintf('Loading data for %d patients...\n', num_patient_files)

% Extract classes from data
classes={};
nfilesTotal=[];
Strs=['AV';'MV';'TV';'PV';'Ph'];
PatientsAV=zeros(1,942);
PatientsMV=zeros(1,942);
PatientsTV=zeros(1,942);
PatientsPh=zeros(1,942);
PatientsPV=zeros(1,942);

OutsAV=-1*ones(1,942);
OutsMV=-1*ones(1,942);
OutsTV=-1*ones(1,942);
OutsPh=-1*ones(1,942);
OutsPV=-1*ones(1,942);

featuresAV=cell(942,1);
featuresMV=cell(942,1);
featuresTV=cell(942,1);
featuresPV=cell(942,1);

featuresPh=cell(100,1);

featuresAVc=cell(942,1);
featuresMVc=cell(942,1);
featuresTVc=cell(942,1);
featuresPVc=cell(942,1);

featuresPhc=cell(100,1);
tic
kav=0;
kpv=0;
ktv=0;
kmv=0;
kph=0;
corrFeatures=[];

for j=1:num_patient_files
    j
    strName=[patient_files{j}(1:end-4),'*.wav'];
    
    current_header=get_header(fullfile(input_directory,patient_files{j}));
    current_recordings=load_recordings(input_directory,current_header);
    
    current_features=get_features(current_header, current_recordings);
    
    %     cpatient_file=dir(fullfile(input_directory);
    cpatient_files=dir(fullfile(input_directory,strName));
    nfiles=length(cpatient_files);
    current_class=get_class(fullfile(input_directory,patient_files{j}));
    current_class
    zclass=0;
    if strcmp(current_class,'Present')==1
        zclass=1;
    else
        if strcmp(current_class,'Unknown')==1
            zclass=2;
        end
    end
    for k=1:nfiles
        cfile=[input_directory,cpatient_files(k).name];
        %         features=challenge(cfile);
        [PCG, Fs1]         = audioread(cfile);  % load data
        PCG_resampled      = resample(PCG,1000,Fs1);
        if strfind(cpatient_files(k).name,'AV')
            kav=kav+1;
            featuresAV{kav}=PCG_resampled;
            OutsAV(kav)=zclass;
            PatientsAV(kav)=str2num( patient_files{j}(1:end-4));
            featuresAVc{kav}=myAutoCorrelationSet(PCG_resampled);
        end
        if strfind(cpatient_files(k).name,'TV')
            ktv=ktv+1;
            featuresTV{ktv}=PCG_resampled;
            OutsTV(ktv)=zclass;
            PatientsTV(ktv)=str2num( patient_files{j}(1:end-4));
            featuresTVc{ktv}=myAutoCorrelationSet(PCG_resampled);
        end
        if strfind(cpatient_files(k).name,'MV')
            kmv=kmv+1;
            featuresMV{kmv}=PCG_resampled;
            OutsMV(kmv)=zclass;
            PatientsMV(kmv)=str2num(patient_files{j}(1:end-4));
            featuresMVc{kmv}=myAutoCorrelationSet(PCG_resampled);
            
        end
        if strfind(cpatient_files(k).name,'PV')
            kpv=kpv+1;
            featuresPV{kpv}=PCG_resampled;
            OutsPV(kpv)=zclass;
            PatientsPV(kpv)=str2num( patient_files{j}(1:end-4));
            featuresPVc{kpv}=myAutoCorrelationSet(PCG_resampled);
            
        end
        if strfind(cpatient_files(k).name,'Ph')
            kph=kph+1;
            featuresPh{kph}=PCG_resampled;
            OutsPh(kph)=zclass;
            PatientsPh(kph)=str2num( patient_files{j}(1:end-4));
            featuresPhc{kph}=myAutoCorrelationSet(PCG_resampled);
        end
        
        %         [PCG, Fs1]         = audioread([recordName '.wav']);  % load data
        %         PCG_resampled      = resample(PCG,1000,Fs1);
    end
    
    if nfiles~=4
        [cpatient_files.name," ", num2str(nfiles)]
        zz=1;
    end
    nfilesTotal=[nfilesTotal,nfiles];
    
    classes=unique([classes current_class]);
end

% for j=1:num_patient_files
%     j
%     strName=[patient_files{j}(1:end-4),'*.wav'];
%
%     current_header=get_header(fullfile(input_directory,patient_files{j}));
%     current_recordings=load_recordings(input_directory,current_header);
%
%     current_features=get_features(current_header, current_recordings);
%
%     %     cpatient_file=dir(fullfile(input_directory);
%     cpatient_files=dir(fullfile(input_directory,strName));
%     nfiles=length(cpatient_files);
%     current_class=get_class(fullfile(input_directory,patient_files{j}));
%     current_class
%     zclass=0;
%     if strcmp(current_class,'Present')==1
%         zclass=1;
%     else
%         if strcmp(current_class,'Unknown')==1
%             zclass=2;
%         end
%     end
%     for k=1:nfiles
%         cfile=[input_directory,cpatient_files(k).name];
%         features=challenge(cfile);
%         if strfind(cpatient_files(k).name,'AV')
%             kav=kav+1;
%             featuresAV(kav,:)=[features,current_features];
%             OutsAV(kav)=zclass;
%             PatientsAV(kav)=str2num( patient_files{j}(1:end-4));
%         end
%         if strfind(cpatient_files(k).name,'TV')
%             ktv=ktv+1;
%             featuresTV(ktv,:)=[features,current_features];
%             OutsTV(ktv)=zclass;
%             PatientsTV(ktv)=str2num( patient_files{j}(1:end-4));
%         end
%         if strfind(cpatient_files(k).name,'MV')
%             kmv=kmv+1;
%             featuresMV(kmv,:)=[features,current_features];
%             OutsMV(kmv)=zclass;
%             PatientsMV(kmv)=str2num(patient_files{j}(1:end-4));
%         end
%         if strfind(cpatient_files(k).name,'PV')
%             kpv=kpv+1;
%             featuresPV(kpv,:)=[features,current_features];
%             OutsPV(kpv)=zclass;
%             PatientsPV(kpv)=str2num( patient_files{j}(1:end-4));
%         end
%         if strfind(cpatient_files(k).name,'Ph')
%             kph=kph+1;
%             featuresPh(kph,:)=[features,current_features];
%             OutsPh(kph)=zclass;
%             PatientsPh(kph)=str2num( patient_files{j}(1:end-4));
%         end
%
%         %         [PCG, Fs1]         = audioread([recordName '.wav']);  % load data
%         %         PCG_resampled      = resample(PCG,1000,Fs1);
%     end
%
%     if nfiles~=4
%         [cpatient_files.name," ", num2str(nfiles)]
%         zz=1;
%     end
%     nfilesTotal=[nfilesTotal,nfiles];
%
%     classes=unique([classes current_class]);
% end
thetime=toc;

% load('allData');

ff=find(PatientsAV==0);numberAv=ff(1)-1;
ff=find(PatientsTV==0);numberTv=ff(1)-1;
ff=find(PatientsPV==0);numberPv=ff(1)-1;
ff=find(PatientsMV==0);numberMv=ff(1)-1;
ff=find(PatientsPh==0);numberPh=ff(1)-1;
PatientsAV=PatientsAV(1:numberAv);
PatientsPV=PatientsPV(1:numberPv);
PatientsTV=PatientsTV(1:numberTv);
PatientsMV=PatientsMV(1:numberMv);
PatientsPh=PatientsPh(1:numberPh);

OutsAV=OutsAV(1:numberAv);
OutsPV=OutsPV(1:numberPv);
OutsTV=OutsTV(1:numberTv);
OutsMV=OutsMV(1:numberMv);
OutsPh=OutsPh(1:numberPh);

featuresPh=featuresPh(1:numberPh,:);
featuresPV=featuresPV(1:numberPv,:);
featuresTV=featuresTV(1:numberTv,:);
featuresMV=featuresMV(1:numberMv,:);
featuresAV=featuresAV(1:numberAv,:);


featuresPhc=featuresPhc(1:numberPh,:);
featuresPVc=featuresPVc(1:numberPv,:);
featuresTVc=featuresTVc(1:numberTv,:);
featuresMVc=featuresMVc(1:numberMv,:);
featuresAVc=featuresAVc(1:numberAv,:);
featuresPhc=cell2mat(featuresPhc);
featuresPVc=cell2mat(featuresPVc);
featuresMVc=cell2mat(featuresMVc);
featuresAVc=cell2mat(featuresAVc);
featuresTVc=cell2mat(featuresTVc);
zz=corrcoef([featuresAVc,OutsAV']);
zz1=zz(1:end-1,end);
[max(zz1),min(zz1)]
zz1'
%  0.1165   -0.1031
['thetime=',num2str(thetime)]
classes=sort(classes);
num_classes=length(classes);
save('signalsResampled')
% Extracting features and labels
disp('Extracting features and labels...')

% features=[];
% labels=categorical;

% for j=1:num_patient_files
%
%     fprintf('%d/%d \n',j,num_patient_files)
%
%     current_header=get_header(fullfile(input_directory,patient_files{j}));
%     current_recordings=load_recordings(input_directory,current_header);
%
%     current_features=get_features(current_header, current_recordings);
%     features(j,:) = current_features(:);
%
%     labels(j)=get_class(fullfile(input_directory,patient_files{j}));
%
% end

%% train RF

disp('Training the model...')

rng('default')
t = templateTree('Reproducible',true);
Y=OutsAV';Y(Y>1)=1;
% Mdlav = fitcensemble(featuresAV,Y,'OptimizeHyperparameters','auto','Learners',t, ...
%     'HyperparameterOptimizationOptions',struct('AcquisitionFunctionName','expected-improvement-plus'))
% [outs,scores]=model.predict(featuresAV(2:2:end,:));
% model = TreeBagger(300,features,labels);
model=fitcensemble(featuresAV(1:2:end,:),Y(1:2:end),'Method','Bag','NumLearningCycles',450);
[outs,scores]=model.predict(featuresAV(2:2:end,:));

scores=scores(:,2);
save_model(model,classes,output_directory);

disp('Done.')

end

function save_model(model,classes,output_directory) %save_PCG_model
% Save results.
filename = fullfile(output_directory,'model.mat');
save(filename,'model','classes','-v7.3');

disp('Done.')
end

function class=get_class(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Murmur'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function current_header=get_header(input_header)

current_header=fileread(input_header);
current_header=strsplit(current_header,'\n');

end

function current_recordings=load_recordings(input_directory,current_header)

recording_files=get_recording_files(current_header);

current_recordings={};

for j=1:length(recording_files)
    
    current_recordings{j}=audioread(fullfile(input_directory,strtrim(recording_files{j})));
    
end

end

function recording_files=get_recording_files(current_header)

recording_files={};

num_locations=strsplit(current_header{1},' ');
num_locations=str2double(num_locations{2});

for j=2:num_locations+1
    
    current_line=strsplit(current_header{j},' ');
    recording_files{j-1}=current_line{3};
    
end

end
